/********************************************************************************/
/*!
	@file			main.c
	@author         Nemui Trinomius (http://nemuisan.blog.bai.ne.jp)
    @version        Vx.xx
    @date           2012.07.10
	@brief          BARE-METAL Project for STM32F407VGT6(STM32F4Discovery).	@n
									   and STM32F407ZGT6(REDBULL).

    @section HISTORY
		2012.07.10	Vx.xx	See Whatnew.txt

    @section LICENSE
		BSD License. See Copyright.txt
*/
/********************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"

/* Defines -------------------------------------------------------------------*/

/* Variables -----------------------------------------------------------------*/

/* Constants -----------------------------------------------------------------*/

/* Function prototypes -------------------------------------------------------*/

/* Functions -----------------------------------------------------------------*/

/**************************************************************************/
/*! 
    @brief  Main Program.
	@param  None.
    @retval None.
*/
/**************************************************************************/
int main(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Set Basis System */
	Set_System();

	/* LED Indicator */
	LED_Configuration();
	
	/* Set SysTickCounter for _delay_ms() and _delay_us() */
	SysTickInit(INTERVAL);

	/* Display Configuration */
	//Display_init_if();

#ifdef USE_REDBULL
	/* RTC Configuration */
	RTC_Configuration();
#endif

	/* Set UART and redirect to STDIO */
	conio_init(UART_DEFAULT_NUM,UART_BAUDLATE);
	/* STDIO function test */
	printf("\n");
	printf("Welcome to %s test program !!\n",MPU_SUBMODEL);
	printf("Version %s!!\n",APP_VERSION);
	printf("Build Date : %s\n",__DATE__);
	printf("\n");

	//PORTDにクロックの供給を開始
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

	GPIO_StructInit(&GPIO_InitStructure);
	//設定するピンを指定する（４つのLED）
	GPIO_InitStructure.GPIO_Pin =
	  GPIO_Pin_12 |GPIO_Pin_13 | GPIO_Pin_14 |GPIO_Pin_15;
	//指定したピンを出力に指定する
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	//出力ポートのタイプをプッシュプルに指定する
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	//プルアップを使用しない
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIOのスピードを100MHz（最高速）にセットする
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;

	GPIO_Init(GPIOD, &GPIO_InitStructure);
	/* Main Loop */
	while (1)
	{
	  GPIO_ResetBits(GPIOD,GPIO_Pin_12);
	  GPIO_ResetBits(GPIOD,GPIO_Pin_13);
	  GPIO_ResetBits(GPIOD,GPIO_Pin_14);
	  GPIO_ResetBits(GPIOD,GPIO_Pin_15);
	  _delay_ms(1000);
	  GPIO_SetBits(GPIOD,GPIO_Pin_12);
	  GPIO_SetBits(GPIOD,GPIO_Pin_13);
	  GPIO_SetBits(GPIOD,GPIO_Pin_14);
	  GPIO_SetBits(GPIOD,GPIO_Pin_15);
	  _delay_ms(1000);
	}

}

/* End Of File ---------------------------------------------------------------*/
